/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/src/net/beandocument/BeanCollectionElement.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-03-25 14:52:56 +0100 (Fr, 25 Mrz 2005) $
** Revision .........: $LastChangedRevision: 46 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.w3c.dom.NodeList;

/**
 * Implements Element for element lists
 * @author Jens Scheffler
 */
public class BeanCollectionElement extends BeanElement {
    
    /** tag name for collection content elements */
    private static final String ITEM_ELEMENT_NAME = "item";

    /** Data Collection for element content */
    private final Collection base;
    
    /** Real base object instance which was used to create the element content */
    private final Object baseObj;
    
    /** List of all child nodes when processed */
    private BeanNodeList childs;

    /**
     * Creates a new collection element
     * @param owner Owner document of this element
     * @param parent Parent node of the new element
     * @param tagName Name of the tag to create
     * @param base Value of the element to use
     */
    public BeanCollectionElement(BeanDocument owner, BeanBaseNode parent, String tagName, Object base) {
        super(owner, parent, tagName, base);

        this.baseObj = base;
        // keep the value and may convert this to a list
        if (base instanceof Collection) {
            this.base = (Collection) base;
        } else if (base instanceof Object[]) {
            Object[] values = (Object[]) base;
            List temp = new Vector(values.length);
            for (int i = 0; i < values.length; i++) {
                temp.add(values[i]);
            }
            this.base = temp;
        } else if (base instanceof Enumeration) {
            Enumeration e = (Enumeration) base;
            List temp = new Vector();
            while (e.hasMoreElements()) {
                temp.add(e.nextElement());
            }
            this.base = temp;
        } else {
            throw new IllegalArgumentException("Given base object for collection element is not of type Collection or Array!");
        }
    }

    /**
     * Retrieves all child nodes of the element
     * @return Child Nodes of this element
     * @see org.w3c.dom.Node#getChildNodes()
     */
    public NodeList getChildNodes() {
        // check if child's have been analyzed already
        if (childs == null) {
            childs = getElementFactory().evaluateChilds(this, base);
            
            // add all collections content items...
            Iterator i = base.iterator();
            BeanElement lastChild = null;
            BeanElementFactory ef = getElementFactory();
            if (childs.getLength() > 0) {
                lastChild = (BeanElement) childs.item(childs.getLength() - 1);
            }
            while (i.hasNext()) {
                // check if no filter applies to content
                Object itemObj = i.next();
                if (ef.filterOK(this, baseObj, ITEM_ELEMENT_NAME, itemObj)) {
                    BeanElement item = ef.getElementForObject(this, ITEM_ELEMENT_NAME, itemObj);
                    childs.add(item);
                    if (lastChild != null) {
                        item.setPreviousSibling(lastChild);
                        lastChild.setNextSibling(item);
                        lastChild = item;
                    }
                }
            }
        }
        
        return childs;
    }
}
