/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/src/net/beandocument/BeanDocument.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-03-17 21:04:48 +0100 (Do, 17 Mrz 2005) $
** Revision .........: $LastChangedRevision: 36 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.EntityReference;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;

/**
 * Wraps a Java bean into a pseudo XML document instance.<br/>
 * This document implementation can be used by the <code>{@link net.beandocument.BeanDocumentBuilder}</code> class
 * which is able to construct new instances of the document by using a simple bean.
 * @author Jens Scheffler
 * @see net.beandocument.BeanDocumentBuilder#getDocument(Object)
 */
public class BeanDocument extends BeanBaseNode implements Document {
    
    /** base bean from which the pseudo DOM document is built from */
    private final Object bean;
    
    /** Document element of this document instance */
    private BeanElement documentElement;
    
    /** Element factory to use for creating new child nodes */
    protected final BeanElementFactory elementFactory;
    
    /**
     * Creates a new document based on the given bean
     * @param elementFactory Element factory to use for all child elements
     * @param bean Java bean to create the document instance from
     */
    protected BeanDocument(BeanElementFactory elementFactory, Object bean) {
        super(null, null);
        
        this.elementFactory = elementFactory;
        this.bean = bean;
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param name Name of the attribute to create - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Document#createAttribute(java.lang.String)
     */
    public Attr createAttribute(String name) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param namespace Namespace for the attribute to create - ignored by this implementation
     * @param name Name of the attribute to create - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Document#createAttributeNS(java.lang.String, java.lang.String)
     */
    public Attr createAttributeNS(String namespace, String name) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param content New content of the CDATA section which should be created - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Document#createCDATASection(java.lang.String)
     */
    public CDATASection createCDATASection(String content) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param content New content of the comment section which should be created - ignored by this implementation
     * @return (This function will never return normally)
     * @throws IllegalArgumentException If the method is called
     * @see org.w3c.dom.Document#createComment(java.lang.String)
     */
    public Comment createComment(String content) {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * Creates a new document fragment
     * @return new document fragment instance
     * @see org.w3c.dom.Document#createDocumentFragment()
     */
    public DocumentFragment createDocumentFragment() {
        return new BeanDocumentFragment(this);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param name New element name which should be created - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Document#createElement(java.lang.String)
     */
    public Element createElement(String name) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param namespace Namespace for the element to create - ignored by this implementation
     * @param name New element name which should be created - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Document#createElementNS(java.lang.String, java.lang.String)
     */
    public Element createElementNS(String namespace, String name) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param name New entity reference name which should be created - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Document#createEntityReference(java.lang.String)
     */
    public EntityReference createEntityReference(String name) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param target New processing instruction target - ignored by this implementation
     * @param data Data for processing instruction - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Document#createProcessingInstruction(java.lang.String, java.lang.String)
     */
    public ProcessingInstruction createProcessingInstruction(String target, String data) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param text New text to use for node - ignored by this implementation
     * @return (This function will never return normally)
     * @throws IllegalArgumentException If the method is called
     * @see org.w3c.dom.Document#createTextNode(java.lang.String)
     */
    public Text createTextNode(String text) {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }

    /**
     * Retrieves all attributes of the current document
     * This will always return null since the document itself has not attributes
     * @return null value
     * @see org.w3c.dom.Node#getAttributes()
     */
    public NamedNodeMap getAttributes() {
        return null;
    }

    /**
     * Retrieves all child nodes of the current element
     * @return List of all child nodes
     * @see org.w3c.dom.Node#getChildNodes()
     */
    public NodeList getChildNodes() {
        if (owner == null) {
            owner = this;
        }
        BeanNodeList result = new BeanNodeList();
        result.add(getDocumentElement());
        return result;
    }

    /**
     * Returns null as document type - beans to not define a specific document type
     * @return null value
     * @see org.w3c.dom.Document#getDoctype()
     */
    public DocumentType getDoctype() {
        return null;
    }

    /**
     * Returns the bean documents root element
     * @return root element of this document
     * @see org.w3c.dom.Document#getDocumentElement()
     */
    public Element getDocumentElement() {
        if (documentElement == null) {
            String name = bean.getClass().getName();
            if (name.indexOf(".") > 0) {
                name = name.substring(name.lastIndexOf(".") + 1);
            }
            while (name.indexOf("$") != -1) {
                name = name.replace('$', '_');
            }
            documentElement = elementFactory.getElementForObject(this, name, bean);
        }
        return documentElement;
    }

    /**
     * Tries to retrieve the element with the given ID - this feature is not implemented!
     * @param id ID to search for in the given document
     * @return Always null, since this method is not implemented
     * @see org.w3c.dom.Document#getElementById(java.lang.String)
     */
    public Element getElementById(String id) {
        return null;
    }

    /**
     * NOTE: This functionality is not implemented since it could lead to infinite crawls.
     * Calls to this method will always return a empty list!
     * @param name Tag name to search for in the document
     * @return Empty NodeList implementation
     * @see org.w3c.dom.Document#getElementsByTagName(java.lang.String)
     */
    public NodeList getElementsByTagName(String name) {
        return new BeanNodeList();
    }

    /**
     * NOTE: This functionality is not implemented since it could lead to infinite crawls.
     * Calls to this method will always return a empty list!
     * @param namespace Namespace to search for in the document
     * @param name Tag name to search for in the document
     * @return Empty NodeList implementation
     * @see org.w3c.dom.Document#getElementsByTagNameNS(java.lang.String, java.lang.String)
     */
    public NodeList getElementsByTagNameNS(String namespace, String name) {
        return new BeanNodeList();
    }

    /**
     * Returns the first child of the document - which matches the root element
     * @return First child of the document
     * @see org.w3c.dom.Node#getFirstChild()
     */
    public Node getFirstChild() {
        return getDocumentElement();
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @return (This function will never return normally)
     * @throws IllegalArgumentException If the method is called
     * @see org.w3c.dom.Document#getImplementation()
     */
    public DOMImplementation getImplementation() {
        throw new IllegalArgumentException("This method is not implemented, please do not call!");
    }

    /**
     * Returns the last child of the document - which matches the root element
     * @return Last child of the document
     * @see org.w3c.dom.Node#getLastChild()
     */
    public Node getLastChild() {
        return getDocumentElement();
    }

    /**
     * Retrieves the local name - since Document has no name, will always return null
     * @return null value
     * @see org.w3c.dom.Node#getLocalName()
     */
    public String getLocalName() {
        return null;
    }

    /**
     * Retrieves the next sibling - since Document itself has no siblings, will always return null
     * @return null value
     * @see org.w3c.dom.Node#getNextSibling()
     */
    public Node getNextSibling() {
        return null;
    }

    /**
     * Returns the name of the Document node, always the fixed "#document" value
     * @return String "#document"
     * @see org.w3c.dom.Node#getNodeName()
     */
    public String getNodeName() {
        return "#document";
    }

    /**
     * Returns node type of this instance
     * @return Node type constant from Node.DOCUMENT_NODE
     * @see org.w3c.dom.Node#getNodeType()
     */
    public short getNodeType() {
        return Node.DOCUMENT_NODE;
    }

    /**
     * Retrieves the node value - since Document itself has no value, will always return null
     * @return null value
     * @throws org.w3c.dom.DOMException (Not really, just returns null)
     * @see org.w3c.dom.Node#getNodeValue()
     */
    public String getNodeValue() throws DOMException {
        return null;
    }

    /**
     * Retrieves the owner document - since Document itself is a document, will always return null
     * @return null value
     * @see org.w3c.dom.Node#getOwnerDocument()
     */
    public Document getOwnerDocument() {
        return null;
    }

    /**
     * Retrieves the parent node - since Document has no parent, will always return null
     * @return null value
     * @see org.w3c.dom.Node#getParentNode()
     */
    public Node getParentNode() {
        return null;
    }

    /**
     * Retrieves the previous sibling - since Document itself has no siblings, will always return null
     * @return null value
     * @see org.w3c.dom.Node#getPreviousSibling()
     */
    public Node getPreviousSibling() {
        return null;
    }

    /**
     * Boolean value if the document has attributes
     * @return Always false - since a document has no attributes
     * @see org.w3c.dom.Node#hasAttributes()
     */
    public boolean hasAttributes() {
        return false;
    }

    /**
     * Boolean value if the document has child nodes
     * @return Flag depending on bean context
     * @see org.w3c.dom.Node#hasChildNodes()
     */
    public boolean hasChildNodes() {
        return (getChildNodes().getLength() > 0);
    }

    /**
     * NOTE: This function is not implemented and therefore will just throw an exception!
     * @param node Node to import - ignored by this implementation
     * @param recursive flag if the node should be imported recursively - ignored by this implementation
     * @return (This function will never return normally)
     * @throws org.w3c.dom.DOMException If method is called
     * @see org.w3c.dom.Document#importNode(org.w3c.dom.Node, boolean)
     */
    public Node importNode(Node node, boolean recursive) throws DOMException {
        throw new DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR, MODIFICATION_ERROR_MESSAGE);
    }
}
