/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/src/net/beandocument/BeanDocumentFragment.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-02-27 15:11:14 +0100 (So, 27 Feb 2005) $
** Revision .........: $LastChangedRevision: 11 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import org.w3c.dom.DOMException;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Document Fragment implementation for bean
 * @author Jens Scheffler
 */
public class BeanDocumentFragment extends BeanBaseNode implements DocumentFragment {

    /**
     * Constructor for a new document fragment
     * @param owner Owner document
     */
    protected BeanDocumentFragment(BeanDocument owner) {
        super(owner, null);

        this.owner = owner;
    }

    /**
     * Fixed node type of this document fragment
     * @return fixed value of Node.DOCUMENT_FRAGMENT_NODE
     * @see org.w3c.dom.Node#getNodeType()
     */
    public short getNodeType() {
        return Node.DOCUMENT_FRAGMENT_NODE;
    }

    /**
     * Returns false since this fragment has no attributes
     * @return false value
     * @see org.w3c.dom.Node#hasAttributes()
     */
    public boolean hasAttributes() {
        return false;
    }

    /**
     * Returns false since this fragment has no attributes
     * @return false value
     * @see org.w3c.dom.Node#hasChildNodes()
     */
    public boolean hasChildNodes() {
        return false;
    }

    /**
     * Local name of this node
     * @return same like getNodeName()
     * @see org.w3c.dom.Node#getLocalName()
     */
    public String getLocalName() {
        return getNodeName();
    }

    /**
     * Node name of this fragment
     * @return fixed value of "#document-fragment"
     * @see org.w3c.dom.Node#getNodeName()
     */
    public String getNodeName() {
        return "#document-fragment";
    }

    /**
     * Node value of this fragment
     * @return null value
     * @throws DOMException (never)
     * @see org.w3c.dom.Node#getNodeValue()
     */
    public String getNodeValue() throws DOMException {
        return null;
    }

    /**
     * Attributes of the fragment
     * @return null value
     * @see org.w3c.dom.Node#getAttributes()
     */
    public NamedNodeMap getAttributes() {
        return null;
    }

    /**
     * First child node
     * @return null value
     * @see org.w3c.dom.Node#getFirstChild()
     */
    public Node getFirstChild() {
        return null;
    }

    /**
     * Last child node
     * @return null value
     * @see org.w3c.dom.Node#getLastChild()
     */
    public Node getLastChild() {
        return null;
    }

    /**
     * Returns empty node list
     * @return empty node list
     * @see org.w3c.dom.Node#getChildNodes()
     */
    public NodeList getChildNodes() {
        return new BeanNodeList();
    }
}
