/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/src/net/beandocument/BeanMapElement.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-03-25 14:52:56 +0100 (Fr, 25 Mrz 2005) $
** Revision .........: $LastChangedRevision: 46 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import java.util.Iterator;
import java.util.Map;

import org.w3c.dom.NodeList;

/**
 * Implements Element for element maps
 * @author Jens Scheffler
 */
public class BeanMapElement extends BeanElement {

    /** tag name for maps content elements */
    private static final String ELEMENT_ELEMENT_NAME = "element";

    /** Name of additional attribute for map key */
    private static final String KEY_ATTRIBUTE_NAME = "key";

    /** Data Map for element content */
    private final Map bean;

    /** List of all child nodes when processed */
    private BeanNodeList childs;

    /**
     * Creates a new collection element
     * @param owner Owner document of this element
     * @param parent Parent node of the new element
     * @param tagName Name of the tag to create
     * @param bean Value of the element to use
     */
    public BeanMapElement(BeanDocument owner, BeanBaseNode parent, String tagName, Object bean) {
        super(owner, parent, tagName, bean);

        if (!(bean instanceof Map)) {
            throw new IllegalArgumentException("Given base object for map element is not of type Map!");
        }
        this.bean = (Map) bean;
    }

    /**
     * Retrieves all child nodes of the element
     * @return Child Nodes of this element
     * @see org.w3c.dom.Node#getChildNodes()
     */
    public NodeList getChildNodes() {
        // check if child's have been analyzed already
        if (childs == null) {
            childs = getElementFactory().evaluateChilds(this, bean);
            
            // add all collections content items...
            Iterator i = bean.keySet().iterator();
            BeanElement lastChild = null;
            BeanElementFactory ef = getElementFactory();
            if (childs.getLength() > 0) {
                lastChild = (BeanElement) childs.item(childs.getLength() - 1);
            }
            while (i.hasNext()) {
                Object key = i.next();
                if (ef.filterOK(this, bean, ELEMENT_ELEMENT_NAME, key)) {
                    BeanElement item = ef.getElementForObject(this, ELEMENT_ELEMENT_NAME, bean.get(key));
                    // add a synthetic key to the element
                    item.addBeanSpecificAttribute(KEY_ATTRIBUTE_NAME, key.toString());
                    childs.add(item);
                    if (lastChild != null) {
                        item.setPreviousSibling(lastChild);
                        lastChild.setNextSibling(item);
                        lastChild = item;
                    }
                }
            }
        }
        
        return childs;
    }
}
