/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/test/net/beandocument/BeanDocumentTest.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-02-27 15:11:14 +0100 (So, 27 Feb 2005) $
** Revision .........: $LastChangedRevision: 11 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import junit.framework.TestCase;

/**
 * Test Case for BeanElementFactory class.
 * @author Jens Scheffler
 */
public class BeanElementFactoryTest extends TestCase {

    private BeanElementFactory f;
    private BeanElement owner;
    private BeanDocument doc;
    private Object testObj;
    
    /**
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        testObj = new Object();
        f = new BeanElementFactory();
        doc = new BeanDocument(f, testObj);
        f.setOwner(doc);
        owner = new BeanSimpleTypeElement(doc, doc, "test", testObj);
    }

    /**
     * A Test Case
     */
    public final void testEvaluateChilds() {
        assertNotNull(f.evaluateChilds(owner, testObj));
        assertTrue(f.evaluateChilds(owner, testObj).getLength() == 0);
    }

    /**
     * A Test Case
     */
    public final void testGetElementForObject() {
        assertTrue(f.getElementForObject(owner, "test", "test") instanceof BeanSimpleTypeElement);
        assertTrue(f.getElementForObject(owner, "test", new Integer(1)) instanceof BeanSimpleTypeElement);
        assertTrue(f.getElementForObject(owner, "test", new Vector()) instanceof BeanCollectionElement);
        assertTrue(f.getElementForObject(owner, "test", testObj) instanceof BeanObjectElement);
        assertTrue(f.getElementForObject(owner, "test", new HashMap()) instanceof BeanMapElement);
    }

    /**
     * A Test Case
     */
    public final void testAddElementMapping() {
        assertTrue(f.getElementForObject(owner, "test", testObj) instanceof BeanObjectElement);
        f.addElementMapping(testObj.getClass(), BeanSimpleTypeElement.class);
        assertTrue(f.getElementForObject(owner, "test", testObj) instanceof BeanSimpleTypeElement);
    }

    /**
     * A Test Case
     */
    public final void testSetOwner() {
        f.setOwner(doc);
    }

    /**
     * A Test Case
     */
    public final void testSetMaxDepth() {
        f.setMaxDepth(0);
    }

    /**
     * A Test Case
     */
    public final void testAddIncludeFilter() {
        f.setMaxDepth(5);
        List test = new Vector();
        test.add("1");
        test.add(new Integer(2));
        test.add(new Long(3));
        assertEquals(3, f.getElementForObject(doc, "test", test).getElementsByTagName("item").getLength());
        f.addIncludeFilter(new SimpleBeanContentFilter((Class) null, null, Number.class));
        assertEquals(2, f.getElementForObject(doc, "test", test).getElementsByTagName("item").getLength());
    }

    /**
     * A Test Case
     */
    public final void testAddExcludeFilter() {
        f.setMaxDepth(5);
        List test = new Vector();
        test.add("1");
        test.add(new Integer(2));
        test.add(new Long(3));
        assertEquals(3, f.getElementForObject(doc, "test", test).getElementsByTagName("item").getLength());
        f.addExcludeFilter(new SimpleBeanContentFilter((Class) null, null, String.class));
        assertEquals(2, f.getElementForObject(doc, "test", test).getElementsByTagName("item").getLength());
    }
}
