/*
** ---------------------------------------------------------------------------
** SVN Repository ...: $HeadURL: file:///Data/_Privat/Programmieren/Source-Repository/SVN/BeanDocument/trunk/test/net/beandocument/BeanDocumentTest.java $
** Author ...........: $LastChangedBy: jscheffl $
** Modified .........: $LastChangedDate: 2005-02-27 15:11:14 +0100 (So, 27 Feb 2005) $
** Revision .........: $LastChangedRevision: 11 $
** Responsible ......: Jens Scheffler
** Initial version ..: Jens Scheffler
** ---------------------------------------------------------------------------
*/
package net.beandocument;

import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import junit.framework.TestCase;

/**
 * Test Case for BeanElement class.
 * @author Jens Scheffler
 */
public class BeanElementTest extends TestCase {
    
    /**
     * Dummy class implementation to test
     * @author jscheffl
     */
    private class ElementImpl extends BeanElement {

        /**
         * @param owner
         * @param parent
         * @param tagName
         * @param bean
         */
        protected ElementImpl(BeanDocument owner, BeanBaseNode parent, String tagName, Object bean) {
            super(owner, parent, tagName, bean);
        }

        /**
         * @return
         * @see org.w3c.dom.Node#getChildNodes()
         */
        public NodeList getChildNodes() {
            return new BeanNodeList();
        }        
    }
    
    private BeanElement el;
    private BeanElement owner;
    private BeanDocument doc;

    /**
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        doc = new BeanDocument(new BeanElementFactory(), this);
        owner = new BeanSimpleTypeElement(doc, doc, "test", this);
        el = new ElementImpl(doc, owner, "test", this);
    }

    /**
     * A Test Case
     */
    public final void testSetNodeValue() {
        el.setNodeValue("new_value");
    }

    /**
     * A Test Case
     */
    public final void testGetAttribute() {
        assertEquals(el.getAttribute("className"), this.getClass().getName());
    }

    /**
     * A Test Case
     */
    public final void testGetAttributeNode() {
        assertEquals("className", el.getAttributeNode("className").getNodeName());
        assertTrue(el.getAttributeNode("className") instanceof BeanAttribute);
        assertEquals(this.getClass().getName(), el.getAttributeNode("className").getNodeValue());
    }

    /**
     * A Test Case
     */
    public final void testGetAttributeNodeNS() {
        assertEquals("className", el.getAttributeNodeNS(null, "className").getNodeName());
        assertTrue(el.getAttributeNodeNS(null, "className") instanceof BeanAttribute);
        assertEquals(this.getClass().getName(), el.getAttributeNodeNS(null, "className").getNodeValue());
        assertNull(el.getAttributeNodeNS("test", "className"));
    }

    /**
     * A Test Case
     */
    public final void testGetAttributeNS() {
        assertEquals(el.getAttributeNS(null, "className"), this.getClass().getName());
        assertNull(el.getAttributeNS("test", "className"));
    }

    /**
     * A Test Case
     */
    public final void testGetAttributes() {
        assertNotNull(el.getAttributes());
        assertEquals(1, el.getAttributes().getLength());
    }

    /**
     * A Test Case
     */
    public final void testGetElementsByTagName() {
        assertEquals(0, el.getElementsByTagName("test").getLength());
        assertEquals(el.getChildNodes().getLength(), el.getElementsByTagName("*").getLength());
    }

    /**
     * A Test Case
     */
    public final void testGetElementsByTagNameNS() {
        assertEquals(0, el.getElementsByTagNameNS(null, "test").getLength());
        assertEquals(el.getChildNodes().getLength(), el.getElementsByTagNameNS(null, "*").getLength());
        assertEquals(0, el.getElementsByTagNameNS("test", "*").getLength());
    }

    /**
     * A Test Case
     */
    public final void testGetFirstChild() {
        if (el.hasChildNodes()) {
            assertNotNull(el.getFirstChild());
        } else {
            assertNull(el.getFirstChild());
        }
    }

    /**
     * A Test Case
     */
    public final void testGetLastChild() {
        if (el.hasChildNodes()) {
            assertNotNull(el.getLastChild());
        } else {
            assertNull(el.getLastChild());
        }
    }

    /**
     * A Test Case
     */
    public final void testGetLocalName() {
        assertEquals("test", el.getLocalName());
        assertEquals(el.getNodeName(), el.getLocalName());
    }

    /**
     * A Test Case
     */
    public final void testGetNodeName() {
        assertEquals("test", el.getNodeName());
    }

    /**
     * A Test Case
     */
    public final void testGetNodeType() {
        assertEquals(Node.ELEMENT_NODE, el.getNodeType());
    }

    /**
     * A Test Case
     */
    public final void testGetNodeValue() {
        assertNull(el.getNodeValue());
    }

    /**
     * A Test Case
     */
    public final void testGetTagName() {
        assertEquals("test", el.getTagName());
    }

    /**
     * A Test Case
     */
    public final void testHasAttribute() {
        assertTrue(el.hasAttribute("className"));
        assertFalse(el.hasAttribute("xxx"));
    }

    /**
     * A Test Case
     */
    public final void testHasAttributeNS() {
        assertTrue(el.hasAttributeNS(null, "className"));
        assertFalse(el.hasAttributeNS(null, "xxx"));
        assertFalse(el.hasAttributeNS("test", "className"));
    }

    /**
     * A Test Case
     */
    public final void testHasAttributes() {
        assertTrue(el.hasAttributes());
    }

    /**
     * A Test Case
     */
    public final void testHasChildNodes() {
        if (el.getChildNodes().getLength() > 0) {
            assertTrue(el.hasChildNodes());
        } else {
            assertFalse(el.hasChildNodes());
        }
    }

    /**
     * A Test Case
     */
    public final void testRemoveAttribute() {
        try {
            el.removeAttribute("test");
        } catch (DOMException e) {
            return;
        }
        assertTrue(false);
    }

    /**
     * A Test Case
     */
    public final void testRemoveAttributeNode() {
        try {
            el.removeAttributeNode(null);
        } catch (DOMException e) {
            return;
        }
        assertTrue(false);
    }

    /**
     * A Test Case
     */
    public final void testRemoveAttributeNS() {
        try {
            el.removeAttributeNS("test", "test");
        } catch (DOMException e) {
            return;
        }
        assertTrue(false);
    }

    /**
     * A Test Case
     */
    public final void testSetAttribute() {
        try {
            el.setAttribute("test", "test");
        } catch (DOMException e) {
            return;
        }
        assertTrue(false);
    }

    /**
     * A Test Case
     */
    public final void testSetAttributeNode() {
        try {
            el.setAttributeNode(null);
        } catch (DOMException e) {
            return;
        }
        assertTrue(false);
    }

    /**
     * A Test Case
     */
    public final void testSetAttributeNodeNS() {
        try {
            el.setAttributeNodeNS(null);
        } catch (DOMException e) {
            return;
        }
        assertTrue(false);
    }

    /**
     * A Test Case
     */
    public final void testSetAttributeNS() {
        try {
            el.setAttributeNS("test", "test", "test");
        } catch (DOMException e) {
            return;
        }
        assertTrue(false);
    }

    /**
     * A Test Case
     */
    public final void testAddBeanSpecificAttribute() {
        assertEquals(1, el.getAttributes().getLength());
        assertNotNull(el.getAttribute("className"));
        assertNull(el.getAttribute("test"));
        el.addBeanSpecificAttribute("test", "test");
        assertEquals(2, el.getAttributes().getLength());
        assertNotNull(el.getAttribute("className"));
        assertNotNull(el.getAttribute("test"));
    }
}
